/*
                               Game Controller
 */
#ifndef GCTL_H
#define GCTL_H

#include <sys/types.h>
#include <sys/stat.h>
#include "gw.h"


/*
 *	Game controller types:
 */
#define GCTL_CONTROLLER_KEYBOARD	0
#define GCTL_CONTROLLER_JOYSTICK	1

/*
 *	Flags data type:
 */
#define gctl_flags_t	unsigned long

/*
 *	Options for controlled type GCTL_CONTROLLER_JOYSTICK:
 *
 *	Note that GCTL_OPT_JS*_INIT determines number of joystick
 *	data structures allocated.
 */
#define GCTL_OPT_JS0_INIT	(1 << 0)
#define GCTL_OPT_JS1_INIT	(1 << 1)

#define GCTL_OPT_JS0_PITCH	(1 << 16)
#define GCTL_OPT_JS0_BANK	(1 << 17)
#define GCTL_OPT_JS0_HEADING	(1 << 18)
#define GCTL_OPT_JS0_THROTTLE	(1 << 19)
#define GCTL_OPT_JS0_HAT	(1 << 20)
#define GCTL_OPT_JS0_AS_THROTTLE_AND_RUDDER	(1 << 21)

#define GCTL_OPT_JS1_PITCH	(1 << 24)
#define GCTL_OPT_JS1_BANK	(1 << 25)
#define GCTL_OPT_JS1_HEADING	(1 << 26)
#define GCTL_OPT_JS1_THROTTLE	(1 << 27)
#define GCTL_OPT_JS1_HAT	(1 << 28)
#define GCTL_OPT_JS1_AS_THROTTLE_AND_RUDDER	(1 << 29)


/*
 *	Game controller positions structure:
 */
typedef struct {

	int type;

	gctl_flags_t options;

	void *data;
	int total_joysticks;

	time_t last_updated;	/* Time this gctl was last updated in ms. */

	/* Control positions. */
	double	heading,	/* -1.0 to 1.0. */
		pitch,		/* -1.0 to 1.0. */
		bank,		/* -1.0 to 1.0. */
		throttle,	/*  0.0 to 1.0. */
		hat_x,		/* -1.0 to 1.0. */
		hat_y;		/* -1.0 to 1.0. */
	int	heading_kb_state,
		pitch_kb_state,
		bank_kb_state,
		throttle_kb_state,
		hat_x_kb_state,
		hat_y_kb_state;
	/* Keyboard last press time stamps for control positions. */
	time_t	heading_kb_last,
		pitch_kb_last,
		bank_kb_last,
		throttle_kb_last,
		hat_x_kb_last,
		hat_y_kb_last;

        /* Joystick button mappings, indicates which joystick button
	 * number corresponds with which action.
	 */
        int	js0_btn_rotate,		/* Treat bank as heading axis. */
		js0_btn_air_brakes,
		js0_btn_wheel_brakes,
		js0_btn_zoom_in,
		js0_btn_zoom_out,
		js0_btn_hoist_up,
		js0_btn_hoist_down;

	int	js1_btn_rotate,		/* Treat bank as heading axis. */
		js1_btn_air_brakes,
		js1_btn_wheel_brakes,
		js1_btn_zoom_in,
		js1_btn_zoom_out,
		js1_btn_hoist_up,
		js1_btn_hoist_down;

	/* Zoom in and zoom out. */
	int zoom_in_state, zoom_out_state;
	time_t zoom_in_kb_last, zoom_out_kb_last;
	double zoom_in_coeff, zoom_out_coeff;	/* 0.0 to 1.0. */

	/* Hoist up and down. */
	int hoist_up_state, hoist_down_state;
        time_t hoist_up_kb_last, hoist_down_kb_last;
	double hoist_up_coeff, hoist_down_coeff;   /* 0.0 to 1.0. */

	/* Alt, ctrl, and shift (not nessasarly key) states. */
	int alt_state, ctrl_state, shift_state;

	/* Brakes state. */
	int air_brakes_state, wheel_brakes_state;
	time_t air_brakes_kb_last, wheel_brakes_kb_last;
	double air_brakes_coeff, wheel_brakes_coeff;

} gctl_struct;

extern int GCtlInit(gctl_struct *gc, int type, gctl_flags_t options);
extern void GCtlUpdate(gctl_struct *gc);
extern void GCtlHandleKey(
        gw_display_struct *display,
        gctl_struct *gc,  
        int k, int kstate,
        int alt_state, int ctrl_state, int shift_state,
	long t
);
extern void GCtlShutdown(gctl_struct *gc);

#endif	/* GCTL_H */
